require 'hashie'
# 启动项目
def run_projects
  fetch_config_files(project_roots).each do |root_path|
    project(root_path, ENV['RACK_ENV'] || 'development')
  end
end

# 启动单独项目
def project(root_path, env = :development)
  config = load_config(root_path, env)
  return unless config[:app_name]

  Eye.application config[:app_name] do
    env 'RAILS_ENV' => env,
        'RACK_ENV' => env,
        'RAILS_ROOT' => root_path,
        'RACK_ROOT' => root_path

    working_dir root_path

    process_global_app(self)

    process_puma(self, config[:puma]) if config[:puma]
    process_sneaker(self, config[:sneaker]) if config[:sneaker]

    group :realtime do
      process_realtime(self, pool: config[:realtime][:pool]) if config[:realtime][:pool]
      process_rt_client(self) if config[:realtime][:rt_client]
    end if config[:realtime]

    group :resque do
      process_scheduler(self) if config[:resque][:scheduler]
      process_worker(self, config[:resque][:worker]) if config[:resque][:worker]
    end if config[:resque]
  end
end

private

def project_roots
  [ENV['REST_SERVER_ROOT'], ENV['TRADING_SERVER_ROOT'], ENV['RAILS_ROOT']]
end

def load_config(root_path, env = :development)
  Hashie.symbolize_keys(YAML.load_file(File.join(root_path, 'config', 'project.yml'))[env.to_s])
end

def fetch_config_files(project_roots = [])
  project_roots << ENV['CURRENT_PWD']
  project_roots.compact.uniq.find_all{|project_root| Eye::Logger.info('='*100 + File.join(project_root, 'config', 'project.yml')); File.exist?(File.join(project_root, 'config', 'project.yml'))}
end
