require 'thor'
module Cs
  module Service
    HOME_PATH = File.realpath(File.join(File.dirname(__FILE__), '..', '..', '..'))

    class Eye < ::Eye::Cli
    end

    # Cli
    class Cli < Thor # rubocop:disable ClassLength
      ENVS = %w(REST_SERVER_ROOT TRADING_SERVER_ROOT RAILS_ROOT RACK_ENV).freeze
      PROJECTS = %w( rest_server trading_server web).freeze
      CONFIG_FILES = {
        web: ENV['RAILS_ROOT'],
        trading_server: ENV['TRADING_SERVER_ROOT'],
        rest_server: ENV['REST_SERVER_ROOT']
      }

      desc 'load', '首次加载配置文件'
      method_option :force, type: :boolean, aliases: '-f', desc: '强制重新加载配置文件'
      def load_conf
        invoke(:env)
        quit if options[:force]
        run_eye(:load, File.join(HOME_PATH, 'config/caishuo.eye'))
      end

      desc 'env [ENV_NAME]', '环境变量检查'
      long_desc <<-LONGDESC
        请声明以下环境变量设置:\n
        REST_SERVER_ROOT
        TRADING_SERVER_ROOT
        RAILS_ROOT
        RACK_ENV
      LONGDESC
      def env(env_name = nil)
        if env_name.nil?
          print_env
        elsif ENV[env_name]
          puts "#{env_name}=#{ENV[env_name]}"
        else
          say "#{env_name} is blank", :yellow
        end
        say "\nConfig Files:"
        fetch_config_files
        say "详见 cs-service help env"
      end

      desc 'info [PROJECT]', '查看当前运行状态'
      def info(name = nil)
        # invoke(:env)
        run_eye(:info, name)
      end
      map status: :info

      {
        start: '启动',
        stop: '停止',
        restart: '重启',
        delete: '删除'
      }.each do |command, name|
        desc "#{command} PROJECT[,...]", "#{name}项目"
        define_method(command) do |*projects|
          projects = PROJECTS if projects == []
          run_eye(command, *projects)
        end
      end

      desc 'quit', '退出 cs-service'
      def quit
        stop
        run_eye(:quit)
      end

      desc 'log [PROJECT]', '查看日志'
      def log(project = nil)
        run_eye(:trace, project || PROJECTS)
      end

      desc 'help [COMMAND]', '帮助'
      def help(command = nil, subcommand = false)
        say Motd.msg unless command
        super(command, subcommand)
      end

      desc 'version', '当前版本'
      def version
        say Cs::Service::VERSION, :green
      end

      desc 'eye ...ARGS', '执行eye口令, 详细参考 cs-service eye'
      subcommand 'eye', Eye

      private

      def print_env
        say("Env:  #{rack_env}", :green)
        ENVS.each do |e|
          puts "  #{e}=#{ENV[e]}"
        end
      end

      def run_eye(command, *args)
        ENV['CURRENT_PWD'] = current_root
        ::Eye::Cli.new.send(command, *args)
      end

      # 查找有效的配置文件
      def fetch_config_files
        config_files = CONFIG_FILES.values.compact.uniq.map do |file|
          check_config_file(file)
        end.compact
        return config_files unless config_files == []
        check_config_file(current_root).nil? ? [] : File.join(current_root, 'config', 'project.yml')
      end

      # 判断当个配置文件是否存在
      def check_config_file(file_root)
        file = File.join(file_root, 'config', 'project.yml')
        if File.exist?(file)
          say("Found: #{file}", :green)
          file
        else
          say("Not Found: #{file}", :red)
          nil
        end
      end

      # 当前root
      def current_root
        File.expand_path('.')
      end

      # 当前运行环境
      def rack_env
        ENV['RACK_ENV'] || 'development'
      end
    end
  end
end
